package util;

import com.alibaba.fastjson.JSONObject;
import java.io.IOException;
import java.security.MessageDigest;
import java.util.*;

import java.io.PrintWriter;

public class ApitUtil {

    //获取应用凭证
    public static String getToken() throws  IOException{
        if(ConstantConfig.TOKEN.length() == 0){
            String res = PostAndGetToken();
            Map stringToMap =  JSONObject.parseObject(res);
            Object accsee_token = stringToMap.get("access_token");
            return (String)accsee_token;
        }

        return  ConstantConfig.TOKEN;
    }

    /**
     * 计算签名
     *
     * @param map
     * @return
     */
    public static String calSign(Map<String, String> map) {
        String str = "";
        Collection<String> keyset= map.keySet();
        List<String> list = new ArrayList<String>(keyset);

        //对key键值按字典升序排序
        Collections.sort(list);
        for (int i = 0; i < list.size(); i++) {
            str += list.get(i) + "=" + map.get(list.get(i)) + "&";
        }

        str += "key=" + ConstantConfig.APP_SECRET;
        System.out.println(str);
        return md5Encode(str).toUpperCase();
    }

    /**
     * MD5
     *
     * @param srcString
     * @return
     */
    public final static String md5Encode(String srcString) {
        char hexDigits[] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
        try {
            byte[] strTemp = srcString.getBytes("UTF-8");
            MessageDigest messageDigest = MessageDigest.getInstance("MD5");
            messageDigest.update(strTemp);
            byte[] mdByte = messageDigest.digest();
            int j = mdByte.length;
            char str[] = new char[j * 2];
            int k = 0;
            for (int i = 0; i < j; i++) {
                byte byte0 = mdByte[i];
                str[k++] = hexDigits[byte0 >>> 4 & 0xf];
                str[k++] = hexDigits[byte0 & 0xf];
            }
            return new String(str);
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * 验证签名
     *
     * @param map
     * @return
     */
    public static Map<String, String> verifySign(Map<String, String> map) {
        String signature = map.remove("signature");

        // 计算签名
        String calSign = calSign(map);

        Map<String, String> requestMap = new TreeMap<String, String>();

        /** 验证签名 */
        if (calSign.equals(signature)) {
            Long localTime = Long.parseLong(String.valueOf(System.currentTimeMillis()).toString().substring(0, 10));
            Long wxTime = Long.parseLong((map.get("timestamp")));
            long timeS = localTime - wxTime;
            /** 检查时间戳 */
            if (timeS >= 0 && timeS < 10) {
                requestMap.put("errcode", "0");
                requestMap.put("errmsg", "");
            } else {
                requestMap.put("errcode", "5003");
                requestMap.put("errmsg", "请求接口失败");
            }
        } else {
            requestMap.put("errcode", "5004");
            requestMap.put("errmsg", "签名错误");
        }

        return requestMap;
    }


    //获取应用凭证
    public static String PostAndGetToken() throws  IOException{
        Map<String, String> params = new HashMap<>();
        params.put("app_key",ConstantConfig.APP_KEY);
        params.put("app_secret",ConstantConfig.APP_SECRET);
        params.put("grant_type","client_credentials");
        params.put("scope","base");
        params.put("ocode",ConstantConfig.O_CODE);
        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.OAUTH_TOKEN_URL);
    }

    //获取主体信息
    public static String getSchoolInfo(String token) throws  IOException{
        Map<String, String> params = new HashMap<>();
        params.put("access_token",token);
        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.SCHOOL_INFO);
    }

    //获取组织架构
    public static String getOrg(String token) throws  IOException{
        Map<String, String> params = new HashMap<>();
        params.put("access_token",token);
        params.put("page","1");
        params.put("page_size","10");
        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.ORG);
    }

    //获取组织架构类型
    public static String getOrgType(String token) throws  IOException{
        Map<String, String> params = new HashMap<>();
        params.put("access_token",token);
        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.ORG_TYPE);
    }

    //获取单个组织架构成员
    public static String getOrgMember(String token) throws  IOException{
        Map<String, String> params = new HashMap<>();
        params.put("access_token",token);
        params.put("org_id",token);
        params.put("fetch_child","0");
        params.put("page","1");
        params.put("page_size","10");
        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.SINGLE_ORG_MEMBER);
    }

    //批量获取成员
    public static String getUserByCardNumber(String token) throws  IOException{
        Map<String, String> params = new HashMap<>();
        String [] car_numbers = new String[1];
        car_numbers[0] = "REAL4411902";
        params.put("access_token",token);
        params.put("card_numbers", HttpUtil.arrayToJson(car_numbers));
        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.getUserByCardNumber);
    }

    //获取成员身份类型
    public static String getIdentType(String token) throws  IOException{
        Map<String, String> params = new HashMap<>();
        params.put("access_token",token);
        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.IDENTITY_TYPE);
    }

    //搜索成员
    public static String getSearchMember(String token) throws  IOException{
        Map<String, String> params = new HashMap<>();
        params.put("access_token",token);
        params.put("keywords","");
        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.SEARCH_MEMBER);
    }

    //获取权限组
    public static String getGroupPermission(String token) throws  IOException{
        Map<String, String> params = new HashMap<>();
        params.put("access_token",token);
        params.put("page","1");
        params.put("page_size","10");
        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.GET_GROUP_PERMISSION);
    }

    //获取权限组详情
    public static String getPermissionDetail(String token) throws  IOException{
        Map<String, String> params = new HashMap<>();
        params.put("access_token",token);
        params.put("group_id","916");
        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.PERMISSION_DETAIL);
    }

    //获取用户权限
    public static String getUserPermission(String token) throws  IOException{
        Map<String, String> params = new HashMap<>();
        params.put("access_token",token);
        params.put("card_number","51447");
        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.GET_USER_PERMISSION);
    }

    //获取角色组
    public static String getRoleGroup(String token) throws  IOException{
        Map<String, String> params = new HashMap<>();
        params.put("access_token",token);
        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.GET_ROLE_GROUP);
    }

    //获取角色
    public static String getRole(String token) throws  IOException{
        Map<String, String> params = new HashMap<>();
        params.put("access_token",token);
        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.GET_ROLE);
    }

    //获取通知
    public static String getNoticeAbility() throws  IOException{
        Map<String, String> params = new HashMap<>();
        long nowTime = System.currentTimeMillis() / 1000L;
        String timeStamp = Long.toString(nowTime);
        params.put("school_code",ConstantConfig.O_CODE);
        params.put("app_key",ConstantConfig.APP_KEY);
        params.put("timestamp",""+timeStamp);
        params.put("nonce","5K8264ILTKCH16CQ2502SI8ZNMTM67VS");
        params.put("signature",calSign(params));
        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.GET_NOTICE_ABILITY);
    }

    //发送通知
    public static String getNoticeSend() throws  IOException{
        Map<String, String> params = new HashMap<>();
        long nowTime = System.currentTimeMillis() / 1000L;
        String timeStamp = Long.toString(nowTime);

        String [] car_numbers = new String[1];
        car_numbers[0] = "REAL4411902";
        params.put("cards", HttpUtil.arrayToJson(car_numbers));
        params.put("school_code",ConstantConfig.O_CODE);
        params.put("title","全体代表大会");
        params.put("content","请大家务必参加");
        params.put("sender","教务处");
        params.put("digest","DIGEST");
        params.put("customs","CUSTOMS");
        params.put("app_key",ConstantConfig.APP_KEY);
        params.put("timestamp",""+timeStamp);
        params.put("nonce","5K8264ILTKCH16CQ2502SI8ZNMTM67VS");
        params.put("signature",calSign(params));
        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.SEND_NOTICE);
    }

    //通知数据明细
    public static String getNoticeDetail() throws  IOException{
        Map<String, String> params = new HashMap<>();
        long nowTime = System.currentTimeMillis() / 1000L;
        String timeStamp = Long.toString(nowTime);
        params.put("mark","MARK");
        params.put("app_key",ConstantConfig.APP_KEY);
        params.put("timestamp",""+timeStamp);
        params.put("nonce","5K8264ILTKCH16CQ2502SI8ZNMTM67VS");
        params.put("signature",calSign(params));
        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.GET_NOTICE_DETAIL);
    }

    //用户阅读通知情况
    public static String getUserRead() throws  IOException{
        Map<String, String> params = new HashMap<>();
        long nowTime = System.currentTimeMillis() / 1000L;
        String timeStamp = Long.toString(nowTime);
        String [] car_numbers = new String[1];
        car_numbers[0] = "REAL4411902";
        params.put("card_number", HttpUtil.arrayToJson(car_numbers));
        params.put("mark","MARK");
        params.put("app_key",ConstantConfig.APP_KEY);
        params.put("timestamp",""+timeStamp);
        params.put("nonce","5K8264ILTKCH16CQ2502SI8ZNMTM67VS");
        params.put("signature",calSign(params));
        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.USER_READ);
    }

    //PC获取用户身份态
    public static String getPcAuth() throws  IOException{
        String redirect_url = "http://test.xdphp.online/app_demo";
        String res = ConstantConfig.PC_CODE+"?app_key="+ConstantConfig.APP_KEY+"&response_type=code&scope=snsapi_sso"+"&school_code="+ConstantConfig.O_CODE+"&redirect_uri="+redirect_url+"&state=pcAuthorize";
        return res;
    }

    //H5获取用户身份态
    public static String getH5Auth() throws  IOException{
        String redirect_url = "http://test.xdphp.online/app_demo";
        String res = ConstantConfig.H5_CODE+"?app_key="+ConstantConfig.APP_KEY+"&response_type=code&scope=snsapi_sso"+"&school_code="+ConstantConfig.O_CODE+"&redirect_uri="+redirect_url+"&state=h5Authorize";
        return res;
    }

    //通过wx_code获取token
    public static String getAccessToken() throws  IOException{
        Map<String, String> params = new HashMap<>();
        String redirect_url = "http://test.xdphp.online/app_demo";

        params.put("wxcode",ConstantConfig.WX_CODE);
        params.put("app_secret",ConstantConfig.APP_SECRET);
        params.put("app_key",ConstantConfig.APP_KEY);
        params.put("grant_type","authorization_code");
        params.put("redirect_uri",redirect_url);

        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.GET_ACCESS_TOKEN);
    }

    //通过Access_token获取授权用户信息
    public static String getUserInfoByAccessToken() throws  IOException{
        Map<String, String> params = new HashMap<>();
        String redirect_url = "http://test.xdphp.online/app_demo";

        params.put("access_token",ConstantConfig.ACCESS_TOKEN);

        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.GET_USER_INFO);
    }

    //电子码解码
    public static String decode() throws  IOException{
        Map<String, String> params = new HashMap<>();
        String redirect_url = "http://test.xdphp.online/app_demo";

        long nowTime = System.currentTimeMillis() / 1000L;
        String timeStamp = Long.toString(nowTime);

        params.put("app_key",ConstantConfig.APP_KEY);
        params.put("timestamp",timeStamp);
        params.put("nonce","5K8264ILTKCH16CQ2502SI8ZNMTM67VS");
        params.put("school_code",ConstantConfig.O_CODE);
        params.put("auth_code","http://wx.url.cn/v001.Eee8m...kA");
        params.put("scene","1");
        params.put("device_no","device-1213");
        params.put("location","三食堂");
        params.put("signature",calSign(params));

        return HttpUtil.executePost(HttpUtil.mapToJson(params),ConstantConfig.CAMPUS_CODE);
    }


}
